import fs from 'fs';
import { createReadStream } from 'fs';
import { createInterface } from 'readline';
import { InsertIncident } from '@shared/schema';

type CSVRow = {
  incidentNumber: string;
  date: string;
  time: string;
  reference: string;
  location: string;
  town: string;
  station: string;
  officerInCharge: string;
  watch: string;
  pumps: string;
  waterCarrier: string;
  supportPump: string;
  aerialLadder: string;
  rescueSupport: string;
  incidentCommand: string;
  incidentType: string;
  notes: string;
  safetyCheck: string;
  additionalNotes: string;
  safetyCheckComplete: string;
};

export async function parseCSV(filePath: string): Promise<Partial<InsertIncident>[]> {
  const fileStream = createReadStream(filePath);
  const rl = createInterface({
    input: fileStream,
    crlfDelay: Infinity,
  });

  const incidents: Partial<InsertIncident>[] = [];
  let isFirstLine = true;
  let headers: string[] = [];
  
  // Define column mappings (expected header names to our schema properties)
  const columnMappings: Record<string, keyof CSVRow> = {
    'id': 'incidentNumber',
    'incidentnumber': 'incidentNumber',
    'incident number': 'incidentNumber',
    'incident_number': 'incidentNumber',
    'number': 'incidentNumber',
    'date': 'date',
    'time': 'time',
    'reference': 'reference',
    'location': 'location',
    'town': 'town',
    'station': 'station',
    'officerInCharge': 'officerInCharge',
    'officer': 'officerInCharge',
    'officer in charge': 'officerInCharge',
    'oic': 'officerInCharge',
    'watch': 'watch',
    'pumps': 'pumps',
    'watercarrier': 'waterCarrier',
    'water carrier': 'waterCarrier',
    'supportpump': 'supportPump',
    'support pump': 'supportPump',
    'aerialladder': 'aerialLadder',
    'aerial ladder': 'aerialLadder',
    'rescuesupport': 'rescueSupport',
    'rescue support': 'rescueSupport',
    'incidentcommand': 'incidentCommand',
    'incident command': 'incidentCommand',
    'incidenttype': 'incidentType',
    'incident type': 'incidentType',
    'type': 'incidentType',
    'notes': 'notes',
    'safetycheck': 'safetyCheck',
    'safety check': 'safetyCheck',
    'additionalnotes': 'additionalNotes',
    'additional notes': 'additionalNotes',
    'safetycheckcomplete': 'safetyCheckComplete',
    'safety check complete': 'safetyCheckComplete'
  };

  for await (const line of rl) {
    // Skip empty lines
    if (!line.trim()) continue;

    try {
      // Handle quotes properly in CSV
      const parts = parseCSVLine(line);
      
      // Process headers in first line
      if (isFirstLine) {
        headers = parts.map(header => header.toLowerCase().trim());
        isFirstLine = false;
        continue;
      }
      
      if (parts.length < 3) continue; // Skip lines with insufficient data
      
      // Map CSV data to our schema using headers
      const row: Partial<CSVRow> = {};
      
      // Set defaults for all fields
      row.incidentNumber = '';
      row.date = '';
      row.time = '';
      row.reference = '';
      row.location = '';
      row.town = '';
      row.station = '';
      row.officerInCharge = '';
      row.watch = '';
      row.pumps = '0';
      row.waterCarrier = '0';
      row.supportPump = '0';
      row.aerialLadder = '0';
      row.rescueSupport = '0';
      row.incidentCommand = '0';
      row.incidentType = '';
      row.notes = '';
      row.safetyCheck = 'False';
      row.additionalNotes = '';
      row.safetyCheckComplete = '';
      
      // Map values from CSV to our schema based on headers
      parts.forEach((value, index) => {
        if (index < headers.length) {
          const header = headers[index];
          const mappedField = columnMappings[header];
          
          if (mappedField) {
            row[mappedField] = value.trim();
          }
        }
      });
      
      // Ensure we have at least number, date and time
      if (!row.incidentNumber) row.incidentNumber = `INC-${Date.now()}`;
      if (!row.date) row.date = new Date().toISOString().split('T')[0];
      if (!row.time) row.time = new Date().toTimeString().split(' ')[0];
      
      incidents.push(row);
    } catch (error) {
      console.error(`Error parsing line: ${line}`, error);
    }
  }

  console.log(`Parsed ${incidents.length} incidents from CSV`);
  return incidents;
}

// Helper function to parse CSV line with proper handling of quotes
function parseCSVLine(line: string): string[] {
  const result: string[] = [];
  let current = '';
  let inQuotes = false;
  
  for (let i = 0; i < line.length; i++) {
    const char = line[i];
    
    if (char === '"') {
      // Toggle quote state
      inQuotes = !inQuotes;
    } else if (char === ',' && !inQuotes) {
      // End of field
      result.push(current);
      current = '';
    } else {
      // Add character to current field
      current += char;
    }
  }
  
  // Add the last field
  result.push(current);
  
  return result;
}
